;----------------------------------------------------------------------------
;     MODULE NAME:   NMAKE.H
;
;         $Author:   USER "Dennis"  $
;       $Revision:   1.0  $
;           $Date:   16 Mar 2001 19:12:28  $
;        $Logfile:   C:/DBAREIS/Projects.PVCS/PpwAddOn/NMAKE/nmake.h.pvcs  $
;
;     DESCRIPTION:   Can create rules for rebuilding files generated via
;                    ppwizard. PPWIZARD has superior dependancy checking to
;                    make programs however there are times when you will
;                    want to use ppwizard as part of a bigger process.
;
;                    When PPWIZARD is used in a make file you will probably
;                    wish to force it to always run and use the
;                    "/dependson:-depfile" format so while it always runs
;                    it does this "quitely" and only generates output on
;                    screen and to the files if required.
;
;                    This header file provides an alternative. It creates
;                    a dependancy file in "make" file format. In your make
;                    file you can conditionally include it, for
;                    example (in nmake syntax):
;
;                         !if exist("out\nmake.inc")
;                         !include <out\nmake.inc>
;                         !endif
;
;                    The first time the above file won't exist and won't get
;                    included but if it doesn't exist the output files
;                    probably won't either and ppwizard will run generating
;                    the files, from then on the make processing can make use
;                    of the rules ppwizard has provided to call ppwizard only
;                    when required.
;
;                    This header assumes no extra files are used or created
;                    in any possible following "onexit" processing. If this
;                    is not the case you should indicate the files to the
;                    header.
;
;                    To make use of this header simply include, indicating
;                    the name of the file to generate.
;
;        WARNINGS:   PPWIZARD can generate warnings in which case ppwizard
;                    would not normally delete the output files. The make
;                    process would then consider the file(s) successfully
;                    built and not restart on subsequent runs. You can use
;                    the /option switch or #option command to set
;                    "warnings=~+*~" if you wish to treat all warnings as
;                    errors.
;
;        FEEDBACK:   As usual feedback is appreciated and will help determine
;                    where future effort is directed.
;----------------------------------------------------------------------------


;--- Error if included twice ------------------------------------------------
#ifdef   VERSION_NMAKE_IH
         #error ^This header has been included twice^
#endif
#define  VERSION_NMAKE_IH    01.075
#require 01.072


;--- Make sure user has indicated the name of the file to create/update -----
#ifndef NMAKE_FILE
        #error ^You must tell me what make include file to create or update via "NMAKE_FILE"^
#endif


;--- Make sure we generate the dependancy information at the end of processing ---
#OnExit <$OnExitNmakeProcessing>


;--- What will we use to MARK start/end of definitions? ---------------------
#define? NMAKE_MARKER <?InputFile>

;--- User want date/time, if so what format? --------------------------------
#define? NMAKE_REXX_DATETIME '### ' || date('W') || ' ' || date('N') || GetAmPmTime()

;--- Processing which we need to do on exit ---------------------------------
#define OnExitNmakeProcessing                          \
        #evaluate ^^ ^<$Rexx4OnExitNmakeProcessing>^
#NextId
#DefineRexx 'Rexx4OnExitNmakeProcessing'
           /*--- We will mark location in file so many can share one file ---*/
           @@Marker  = '[<$NMAKE_MARKER>]'
           @@MarkerS = '### ' || @@Marker || ' START #######################';
           @@MarkerE = '### ' || @@Marker || ' END #########################';

           /*--- If file exists read in existing information ----------------*/
           @@File = '<$NMAKE_FILE>';
           @@Rc = stream(@@File, 'c', 'close');
           if  stream(@@File, 'c', 'query exists') <> '' then
           do
               /*--- File exists, read into memory --------------------------*/
               @@Lines    = 0;
               @@Blanks   = 0;                       ;;Counts blank lines
               @@FndStart = 'N';                     ;;Found start of block?
               do while lines(@@File) <> 0
                  /*--- Read the line (translate EOF char) ------------------*/
                  @@This = linein(@@File);
                  @@This = translate(@@This, ' ', '1A'x);

                  /*--- Dropping duplicate blank lines ----------------------*/
                  if   @@This = '' then
                       @@Blanks = @@Blanks + 1;    ;;Just count blank lines
                  else
                  do
                       /*--- Is this the marker? ----------------------------*/
                       #if ['<?OpSys>' = 'UNIX']
                           /*--- Filenames in unix are case sensitive -------*/
                           if  pos(@@Marker, @@This) <> 0 then
                       #elseif
                           /*--- This operating system has case insenstive filenames ---*/
                           if  pos(translate(@@Marker), translate(@@This)) <> 0 then
                       #endif
                       do
                           /*--- Found start or end marker ------------------*/
                           if  @@FndStart = 'N' then
                               @@FndStart = 'Y';   ;;Found start of block
                           else
                           do
                               @@FndStart = 'N';   ;;Found end of block
                               iterate;            ;;Also ignore end marker
                           end;
                       end;
                       if  @@FndStart = 'Y' then
                           iterate;                ;;Drop lines until end found

                       /*--- Not a blank line, output a blank line if required ---*/
                       if  @@Blanks <> 0 then
                       do
                           @@Lines        = @@Lines + 1;
                           @@Line.@@Lines = '';
                           @@Blanks       = 0;
                       end;

                       /*--- Save the line ----------------------------------*/
                       @@Lines        = @@Lines + 1;
                       @@Line.@@Lines = @@This;
                  end;
               end;

               /*--- Now Delete and recreate the file (without block) -------*/
               call MustDeleteFile @@File;
               do  @@I = 1 to @@Lines
                   call lineout @@File, @@Line.@@I;
               end;
               @@Rc = stream(@@File, 'c', 'close');
           end;

           /*--- Mark start -------------------------------------------------*/
           call lineout @@File, ''
           call lineout @@File, @@MarkerS;
           @@Dt = <$NMAKE_REXX_DATETIME>;
           if  @@Dt <> '' then
               call lineout @@File, @@Dt;      ;;Date/Time is optional

           /*--- Output targets ---------------------------------------------*/
           @@Max     = GetDependancyInfo('OUTPUT');
           @@Longest = 0;
           do  @@I = 1 to @@Max
               /*--- Get value and keep record of longest -------------------*/
               ThisDep = GetDependancyInfo('OUTPUT', @@I);
               if  length(ThisDep) > @@Longest then
                   @@Longest = length(ThisDep);
               ThisDep = left(ThisDep, @@Longest);

               /*--- How does the line end? ---------------------------------*/
               if @@I = @@Max then
               do
                   /*--- Last dependancy ------------------------------------*/
                   @@EOL = ': \'
               end;
               else
               do
                   /*--- Not the last dependancy ----------------------------*/
                   @@EOL = '  \'
               end;
               call lineout @@File, ThisDep || @@EOL;
           end;

           /*--- Output source files ----------------------------------------*/
           @@Max     = GetDependancyInfo('INPUT');
           @@Longest = 0;
           do @@I = 1 to @@Max
               /*--- Get value and pad out to longest (so far) length -------*/
               ThisDep = GetDependancyInfo('INPUT', @@I);
               if  length(ThisDep) > @@Longest then
                   @@Longest = length(ThisDep);
               ThisDep = left(ThisDep, @@Longest);

               /*--- How does the line end? ---------------------------------*/
               if  @@I = @@Max then
               do
                   /*--- Last dependancy ------------------------------------*/
                   @@EOL = ''
               end;
               else
               do
                   /*--- Not the last dependancy ----------------------------*/
                   @@EOL = ' \'
               end;
               call lineout @@File, '    ' || ThisDep || @@EOL;
           end;

           /*--- Mark end ---------------------------------------------------*/
           call lineout @@File, @@MarkerE;
           call lineout @@File, ''
           @@Rc = stream(@@File, 'c', 'close');
#DefineRexx
